package cn.com.qmth.examcloud.reports.commons.util;

import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.reports.commons.bean.BaseReport;
import cn.com.qmth.examcloud.reports.commons.enums.Tag;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.support.IpUtil;
import cn.com.qmth.examcloud.web.support.ServletUtil;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.*;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

public class ReportsUtil {

    private static final Logger LOG = LoggerFactory.getLogger(ReportsUtil.class);

    private static final String GROUP_PRODUCER_PREFIX = "ec_producer_";

    private static final String GROUP_CONSUMER_PREFIX = "ec_consumer_";

    private static final String TOPIC_PREFIX = "ec_topic_";

    private static final String DATA_KEY = "ec_mq_data";

    private static Producer producer;

    private static Boolean reportEnable = PropertyHolder.getBoolean("examcloud.rocketmq.enable", false);

    static {
        if (reportEnable) {
            Properties properties = new Properties();
            // AccessKeyId 阿里云身份验证，在阿里云服务器管理控制台创建。
            properties.put(PropertyKeyConst.AccessKey, PropertyHolder.getString("examcloud.rocketmq.accesskey"));
            // AccessKeySecret 阿里云身份验证，在阿里云服务器管理控制台创建。
            properties.put(PropertyKeyConst.SecretKey, PropertyHolder.getString("examcloud.rocketmq.secretkey"));
            // 设置发送超时时间，单位毫秒。
            properties.setProperty(PropertyKeyConst.SendMsgTimeoutMillis, "3000");
            // 设置 TCP 接入域名，进入控制台的实例详情页面的 TCP 协议客户端接入点区域查看。
            properties.put(PropertyKeyConst.NAMESRV_ADDR, PropertyHolder.getString("examcloud.rocketmq.namesrv_addr"));

            // 生产组名称
            properties.put(PropertyKeyConst.GROUP_ID, curProducerGroup());

            producer = ONSFactory.createProducer(properties);
            // 在发送消息前，必须调用 start 方法来启动 Producer，只需调用一次即可。
            producer.start();
            LOG.warn("Rocketmq Producer started...");
        }
    }

    private static void sendReportRocket(Boolean onException) {
        @SuppressWarnings("unchecked")
        List<BaseReport> list = (List<BaseReport>) ThreadLocalUtil.get(DATA_KEY);
        if (CollectionUtils.isNotEmpty(list)) {
            for (BaseReport b : list) {
                if (!onException || (onException && b.getReportOnException())) {
                    try {
                        String messageStr = JSON.toJSONString(b);
                        Message msg = new Message(b.getTopic(), b.getTag(), messageStr.getBytes("utf-8"));
                        producer.sendAsync(msg, new SendCallback() {

                            @Override
                            public void onSuccess(final SendResult sendResult) {
                                // 消息发送成功。
                            }

                            @Override
                            public void onException(OnExceptionContext context) {
                                // 消息发送失败，需要进行重试处理，可重新发送这条消息或持久化这条数据进行补偿处理。
                                LOG.error("SendReport failed msgId:" + msg + " " + messageStr, context.getException());
                            }
                        });
                    } catch (Exception e) {
                        LOG.error("SendReport Error:" + JSON.toJSONString(b), e);
                    }
                }
            }
            ThreadLocalUtil.set(DATA_KEY, null);
        }
    }

    public static void report(BaseReport report) {
        try {
            if (!reportEnable) {
                return;
            }
            setReportCommonData(report);
            @SuppressWarnings("unchecked")
            List<BaseReport> list = (List<BaseReport>) ThreadLocalUtil.get(DATA_KEY);
            if (list == null) {
                list = new ArrayList<BaseReport>();
                ThreadLocalUtil.set(DATA_KEY, list);
            }
            list.add(report);
        } catch (Exception e) {
            LOG.error(JSON.toJSONString(report), e);
        }
    }

    public static void sendReport(Boolean onException) {
        sendReportRocket(onException);
    }

    private static void setReportCommonData(BaseReport report) {
        String ip = "";
        try {
            InetAddress localHost = Inet4Address.getLocalHost();
            ip = localHost.getHostAddress();
        } catch (Exception e) {
            LOG.debug("获取本机IP出错", e);
        }

        report.setReportHost(ip);
        report.setRemoteHost(IpUtil.getRemoteIp(ServletUtil.getRequest()));
        report.setReportTime(new Date());
    }

    public static String curTopic(Tag tag) {
        return TOPIC_PREFIX + tag.name().toLowerCase() + "_" + curProfile();
    }

    public static String curConsumerGroup(Tag tag) {
        return GROUP_CONSUMER_PREFIX + tag.name().toLowerCase() + "_" + curProfile();
    }

    public static String curProducerGroup() {
        return GROUP_PRODUCER_PREFIX + "_" + curProfile();
    }

    public static String curProfile() {
        return PropertyHolder.getString("spring.profiles.active", "");
    }

}
