package cn.com.qmth.examcloud.support.test;

import cn.com.qmth.examcloud.commons.util.FileUtil;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.support.fss.FssFactory;
import cn.com.qmth.examcloud.support.fss.FssHelper;
import cn.com.qmth.examcloud.support.fss.FssProperty;
import cn.com.qmth.examcloud.support.fss.FssService;
import cn.com.qmth.examcloud.support.fss.model.FssFileInfo;
import cn.com.qmth.examcloud.support.fss.model.FssSignInfo;
import cn.com.qmth.examcloud.support.fss.model.FssType;
import okhttp3.*;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.core.config.Configurator;
import org.junit.Before;
import org.junit.Test;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;

public class OssClientTest {

    @Before
    public void init() {
        Configurator.setRootLevel(Level.INFO);
    }

    @Test
    public void demoTest() throws Exception {
        FssProperty.FSS_TYPE = FssType.ALIYUN_OSS;
        FssProperty.FSS_BUCKET = "examcloud-test";
        FssProperty.FSS_REGION_ID = "oss-cn-shenzhen";
        FssProperty.FSS_ENDPOINT = "https://oss-cn-shenzhen.aliyuncs.com";
        FssProperty.FSS_INTERNAL_ENDPOINT = "https://oss-cn-shenzhen-internal.aliyuncs.com";
        FssProperty.FSS_ACCESS_KEY_ID = "LTAI4FboXLCJzrjVo5dUoXaU";
        FssProperty.FSS_ACCESS_KEY_SECRET = "xxx";
        // FssProperty.FSS_URL_PREFIX = "https://examcloud-test.oss-cn-shenzhen.aliyuncs.com";
        FssProperty.FSS_URL_PREFIX = "https://ecs-test-static.qmth.com.cn";

        // FssProperty.FSS_TYPE = FssType.TENCENT_COS;
        // FssProperty.FSS_BUCKET = "examcloud-1252178304";
        // FssProperty.FSS_REGION_ID = "ap-guangzhou";
        // FssProperty.FSS_ENDPOINT = "https://cos.ap-guangzhou.myqcloud.com";
        // FssProperty.FSS_INTERNAL_ENDPOINT = "https://cos.ap-guangzhou.myqcloud.com";
        // FssProperty.FSS_ACCESS_KEY_ID = "AKID51lO89AEFNRNA90jGZiw9E5kCQh7djpn";
        // FssProperty.FSS_ACCESS_KEY_SECRET = "xxx";
        // FssProperty.FSS_URL_PREFIX = "https://examcloud-1252178304.cos.ap-guangzhou.myqcloud.com";

        final String testDir = "D:/home";
        final String filePath = "test/abc.png";
        File testFile = new File(testDir + "/" + filePath);
        final String testFileMd5 = "80bf42bd4d12b2a38eff3b32249b17fd";

        FssService fssService = FssFactory.getInstance(false);
        // this.upload(fssService, filePath, testFile, testFileMd5);
        // this.signUpload(fssService, filePath, testFile, testFileMd5);
        // this.download(fssService, filePath, testDir);
        // fssService.refreshFile(FssProperty.FSS_URL_PREFIX + "/" + filePath);
    }

    public void upload(FssService fssService, String filePath, File testFile, String testFileMd5) throws Exception {
        byte[] testFileBytes = IOUtils.toByteArray(Files.newInputStream(testFile.toPath()));
        FssFileInfo result = fssService.writeFile(filePath, testFileBytes, testFileMd5);
        System.out.println(result.getFileUrl());
    }

    public void signUpload(FssService fssService, String filePath, File testFile, String testFileMd5) {
        FssSignInfo signInfo = fssService.buildSign(filePath, testFileMd5);
        System.out.println(new JsonMapper().toJson(signInfo));

        MultipartBody.Builder form = new MultipartBody.Builder().setType(MultipartBody.FORM);
        RequestBody reqBody = RequestBody.create(MediaType.parse("application/octet-stream"), testFile);
        form.addFormDataPart("key", signInfo.getFormParams().get("key"));
        form.addFormDataPart("OSSAccessKeyId", signInfo.getFormParams().get("OSSAccessKeyId"));
        form.addFormDataPart("policy", signInfo.getFormParams().get("policy"));
        form.addFormDataPart("Signature", signInfo.getFormParams().get("Signature"));
        form.addFormDataPart("file", testFile.getName(), reqBody);

        Request.Builder request = new Request.Builder().url(signInfo.getFormUrl()).post(form.build());
        try (Response response = new OkHttpClient.Builder().build().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body != null ? body.string() : null;
            if (response.isSuccessful()) {
                String reqMD5 = FssHelper.md5ToBase64(testFileMd5);
                String respMD5 = response.header("Content-MD5");
                System.out.println("上传成功！reqMD5:" + reqMD5 + " respMD5:" + respMD5);
                return;
            }
            System.out.println("上传失败！" + bodyStr);
        } catch (IOException e) {
            System.out.println("上传异常！" + e.getMessage());
        }
    }

    public void download(FssService fssService, String filePath, String testDir) {
        File newFile = new File(testDir + "/test/abc-new.png");
        byte[] newFileBytes = fssService.readFile(filePath);
        FileUtil.saveToFile(newFileBytes, newFile);
    }

    // @Test
    public void demoTest2() throws Exception {
        FssProperty.FSS_URL_PREFIX = "https://ecs-test-static.qmth.com.cn";

        System.out.println(FssHelper.finalFileUrl("upyun-1://abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("aliyun-1://abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("aliyun-2://abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("https://xxx.xxx/abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("http://127.0.0.1:8080/abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("/abc/123.jpg"));
        System.out.println(FssHelper.finalFileUrl("abc/123.jpg"));

        System.out.println(FssHelper.fixFilePath("upyun-1://abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("aliyun-1://abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("aliyun-2://abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("https://xxx.xxx/abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("http://127.0.0.1:8080/abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("/abc/123.jpg"));
        System.out.println(FssHelper.fixFilePath("abc/123.jpg"));
    }

}
